@@ -37,7 +37,6 @@
 
 #include "ev-poppler.h"
 #include "ev-file-exporter.h"
-#include "ev-mapping.h"
 #include "ev-document-find.h"
 #include "ev-document-misc.h"
 #include "ev-document-links.h"
@@ -111,6 +110,7 @@ struct _PdfDocument
 	PdfPrintContext *print_ctx;
 
 	GList *layers;
+	GHashTable *annots;
 };
 
 static void pdf_document_security_iface_init             (EvDocumentSecurityInterface    *iface);
@@ -188,6 +188,11 @@ pdf_document_dispose (GObject *object)
 		pdf_document->print_ctx = NULL;
 	}
 
+	if (pdf_document->annots) {
+		g_hash_table_destroy (pdf_document->annots);
+		pdf_document->annots = NULL;
+	}
+
 	if (pdf_document->document) {
 		g_object_unref (pdf_document->document);
 	}
@@ -1242,7 +1247,7 @@ pdf_document_links_get_links_model (EvDocumentLinks *document_links)
 	return model;
 }
 
-static GList *
+static EvMappingList *
 pdf_document_links_get_links (EvDocumentLinks *document_links,
 			      EvPage          *page)
 {
@@ -1277,7 +1282,7 @@ pdf_document_links_get_links (EvDocumentLinks *document_links,
 
 	poppler_page_free_link_mapping (mapping_list);
 
-	return g_list_reverse (retval);
+	return ev_mapping_list_new (page->index, g_list_reverse (retval), (GDestroyNotify)g_object_unref);
 }
 
 static EvLinkDest *
@@ -1308,7 +1313,7 @@ pdf_document_document_links_iface_init (EvDocumentLinksInterface *iface)
 	iface->find_link_dest = pdf_document_links_find_link_dest;
 }
 
-static GList *
+static EvMappingList *
 pdf_document_images_get_image_mapping (EvDocumentImages *document_images,
 				       EvPage           *page)
 {
@@ -1341,7 +1346,7 @@ pdf_document_images_get_image_mapping (EvDocumentImages *document_images,
 
 	poppler_page_free_image_mapping (mapping_list);
 
-	return g_list_reverse (retval);
+	return ev_mapping_list_new (page->index, g_list_reverse (retval), (GDestroyNotify)g_object_unref);
 }
 
 GdkPixbuf *
@@ -1906,11 +1911,17 @@ pdf_selection_get_selected_text (EvSelection     *selection,
 				 EvRectangle     *points)
 {
 	PopplerPage *poppler_page;
+	char *retval;
+
+	poppler_page = POPPLER_PAGE (page->backend_page);
+
+#ifdef HAVE_POPPLER_PAGE_GET_SELECTED_TEXT
+	retval = poppler_page_get_selected_text (poppler_page,
+						 (PopplerSelectionStyle)style,
+						 (PopplerRectangle *)points);
+#else
 	PopplerRectangle r;
 	double height;
-	char *retval;
-	
-	poppler_page = POPPLER_PAGE (page->backend_page);
 
 	poppler_page_get_size (poppler_page, NULL, &height);
 	r.x1 = points->x1;
@@ -1921,53 +1932,53 @@ pdf_selection_get_selected_text (EvSelection     *selection,
 	retval = poppler_page_get_text (poppler_page,
 					(PopplerSelectionStyle)style,
 					&r);
+#endif /* HAVE_POPPLER_PAGE_GET_SELECTED_TEXT */
 
 	return retval;
 }
 
-static GdkRegion *
-create_gdk_region_from_poppler_region (GList *region)
+static cairo_region_t *
+create_region_from_poppler_region (GList *region, gdouble scale)
 {
 	GList *l;
-	GdkRegion *retval;
-	
-	retval = gdk_region_new ();
-	
+	cairo_region_t *retval;
+
+	retval = cairo_region_create ();
+
 	for (l = region; l; l = g_list_next (l)) {
-		PopplerRectangle *rectangle;
-		GdkRectangle      rect;
-		
+		PopplerRectangle   *rectangle;
+		cairo_rectangle_int_t rect;
+
 		rectangle = (PopplerRectangle *)l->data;
-		
-		rect.x = (gint) rectangle->x1;
-		rect.y = (gint) rectangle->y1;
-		rect.width  = (gint) (rectangle->x2 - rectangle->x1);
-		rect.height = (gint) (rectangle->y2 - rectangle->y1);
-		gdk_region_union_with_rect (retval, &rect);
-		
+
+		rect.x = (gint) ((rectangle->x1 * scale) + 0.5);
+		rect.y = (gint) ((rectangle->y1 * scale) + 0.5);
+		rect.width  = (gint) (((rectangle->x2 - rectangle->x1) * scale) + 0.5);
+		rect.height = (gint) (((rectangle->y2 - rectangle->y1) * scale) + 0.5);
+		cairo_region_union_rectangle (retval, &rect);
+
 		poppler_rectangle_free (rectangle);
 	}
 
 	return retval;
 }
 
-static GdkRegion *
+static cairo_region_t *
 pdf_selection_get_selection_region (EvSelection     *selection,
 				    EvRenderContext *rc,
 				    EvSelectionStyle style,
 				    EvRectangle     *points)
 {
-	PopplerPage *poppler_page;
-	GdkRegion   *retval;
-	GList       *region;
+	PopplerPage    *poppler_page;
+	cairo_region_t *retval;
+	GList          *region;
 
 	poppler_page = POPPLER_PAGE (rc->page->backend_page);
-	
 	region = poppler_page_get_selection_region (poppler_page,
-						    rc->scale,
+						    1.0,
 						    (PopplerSelectionStyle)style,
 						    (PopplerRectangle *) points);
-	retval = create_gdk_region_from_poppler_region (region);
+	retval = create_region_from_poppler_region (region, rc->scale);
 	g_list_free (region);
 	
 	return retval;
@@ -1983,14 +1994,14 @@ pdf_selection_iface_init (EvSelectionInterface *iface)
 
 
 /* EvDocumentText */
-static GdkRegion *
+static cairo_region_t *
 pdf_document_text_get_text_mapping (EvDocumentText *document_text,
 				    EvPage         *page)
 {
 	PopplerPage *poppler_page;
 	PopplerRectangle points;
 	GList *region;
-	GdkRegion *retval;
+	cairo_region_t *retval;
 
 	g_return_val_if_fail (POPPLER_IS_PAGE (page->backend_page), NULL);
 
@@ -2003,12 +2014,26 @@ pdf_document_text_get_text_mapping (EvDocumentText *document_text,
 	region = poppler_page_get_selection_region (poppler_page, 1.0,
 						    POPPLER_SELECTION_GLYPH,
 						    &points);
-	retval = create_gdk_region_from_poppler_region (region);
+	retval = create_region_from_poppler_region (region, 1.0);
 	g_list_free (region);
 
 	return retval;
 }
 
+#ifdef HAVE_POPPLER_PAGE_GET_SELECTED_TEXT
+static gchar *
+pdf_document_text_get_text (EvDocumentText  *selection,
+			    EvPage          *page)
+{
+	PopplerPage *poppler_page;
+
+	g_return_val_if_fail (POPPLER_IS_PAGE (page->backend_page), NULL);
+
+	poppler_page = POPPLER_PAGE (page->backend_page);
+
+	return poppler_page_get_text (poppler_page);
+}
+#else
 static gchar *
 pdf_document_text_get_text (EvDocumentText  *selection,
 			    EvPage          *page)
@@ -2028,6 +2053,7 @@ pdf_document_text_get_text (EvDocumentText  *selection,
 				      POPPLER_SELECTION_WORD,
 				      &r);
 }
+#endif /* HAVE_POPPLER_PAGE_GET_SELECTED_TEXT */
 
 #ifdef HAVE_POPPLER_PAGE_GET_TEXT_LAYOUT
 static gboolean
@@ -2244,7 +2270,7 @@ ev_form_field_from_poppler_field (PopplerFormField *poppler_field)
 	return ev_field;
 }
 
-static GList *
+static EvMappingList *
 pdf_document_forms_get_form_fields (EvDocumentForms *document, 
 				    EvPage          *page)
 {
@@ -2289,7 +2315,9 @@ pdf_document_forms_get_form_fields (EvDocumentForms *document,
 	
 	poppler_page_free_form_field_mapping (fields);
 
-	return g_list_reverse (retval);
+	return retval ? ev_mapping_list_new (page->index,
+					     g_list_reverse (retval),
+					     (GDestroyNotify)g_object_unref) : NULL;
 }
 
 static gchar *
@@ -2587,9 +2615,18 @@ ev_annot_from_poppler_annot (PopplerAnnot *poppler_annot,
 	}
 
 	if (ev_annot) {
+		time_t utime;
+		gchar *modified;
+
 		ev_annot->contents = poppler_annot_get_contents (poppler_annot);
 		ev_annot->name = poppler_annot_get_name (poppler_annot);
-		ev_annot->modified = poppler_annot_get_modified (poppler_annot);
+		modified = poppler_annot_get_modified (poppler_annot);
+		if (poppler_date_parse (modified, &utime)) {
+			ev_annot->modified = ev_document_misc_format_date (utime);
+			g_free (modified);
+		} else {
+			ev_annot->modified = modified;
+		}
 		poppler_annot_color_to_gdk_color (poppler_annot, &ev_annot->color);
 
 		if (POPPLER_IS_ANNOT_MARKUP (poppler_annot)) {
@@ -2643,13 +2680,14 @@ ev_annot_from_poppler_annot (PopplerAnnot *poppler_annot,
 	return ev_annot;
 }
 
-static GList *
+static EvMappingList *
 pdf_document_annotations_get_annotations (EvDocumentAnnotations *document_annotations,
 					  EvPage                *page)
 {
 	GList *retval = NULL;
 	PdfDocument *pdf_document;
 	PopplerPage *poppler_page;
+	EvMappingList *mapping_list;
 	GList *annots;
 	GList *list;
 	gdouble height;
@@ -2657,6 +2695,14 @@ pdf_document_annotations_get_annotations (EvDocumentAnnotations *document_annota
 
 	pdf_document = PDF_DOCUMENT (document_annotations);
 	poppler_page = POPPLER_PAGE (page->backend_page);
+
+	if (pdf_document->annots) {
+		mapping_list = (EvMappingList *)g_hash_table_lookup (pdf_document->annots,
+								     GINT_TO_POINTER (page->index));
+		if (mapping_list)
+			return ev_mapping_list_ref (mapping_list);
+	}
+
 	annots = poppler_page_get_annot_mapping (poppler_page);
 	poppler_page_get_size (poppler_page, NULL, &height);
 
@@ -2694,7 +2740,22 @@ pdf_document_annotations_get_annotations (EvDocumentAnnotations *document_annota
 
 	poppler_page_free_annot_mapping (annots);
 
-	return g_list_reverse (retval);
+	if (!retval)
+		return NULL;
+
+	if (!pdf_document->annots) {
+		pdf_document->annots = g_hash_table_new_full (g_direct_hash,
+							      g_direct_equal,
+							      (GDestroyNotify)NULL,
+							      (GDestroyNotify)ev_mapping_list_unref);
+	}
+
+	mapping_list = ev_mapping_list_new (page->index, g_list_reverse (retval), (GDestroyNotify)g_object_unref);
+	g_hash_table_insert (pdf_document->annots,
+			     GINT_TO_POINTER (page->index),
+			     ev_mapping_list_ref (mapping_list));
+
+	return mapping_list;
 }
 
 static void
