@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include "config.h"
@@ -24,8 +24,9 @@
 #include "ev-annotation.h"
 
 
-static void ev_annotation_markup_iface_base_init (EvAnnotationMarkupIface *iface);
-static void ev_annotation_text_markup_iface_init (EvAnnotationMarkupIface *iface);
+static void ev_annotation_markup_default_init          (EvAnnotationMarkupInterface *iface);
+static void ev_annotation_text_markup_iface_init       (EvAnnotationMarkupInterface *iface);
+static void ev_annotation_attachment_markup_iface_init (EvAnnotationMarkupInterface *iface);
 
 enum {
 	PROP_0,
@@ -37,35 +38,17 @@ enum {
 };
 
 G_DEFINE_ABSTRACT_TYPE (EvAnnotation, ev_annotation, G_TYPE_OBJECT)
-GType
-ev_annotation_markup_get_type (void)
-{
-	static volatile gsize g_define_type_id__volatile = 0;
-
-	if (g_once_init_enter (&g_define_type_id__volatile)) {
-		GType g_define_type_id;
-		const GTypeInfo our_info = {
-			sizeof (EvAnnotationMarkupIface),
-			(GBaseInitFunc) ev_annotation_markup_iface_base_init,
-			NULL,
-		};
-
-		g_define_type_id = g_type_register_static (G_TYPE_INTERFACE,
-							   "EvAnnotationMarkup",
-							   &our_info, (GTypeFlags)0);
-		g_type_interface_add_prerequisite (g_define_type_id, EV_TYPE_ANNOTATION);
-
-		g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
-	}
-
-	return g_define_type_id__volatile;
-}
-
+G_DEFINE_INTERFACE (EvAnnotationMarkup, ev_annotation_markup, EV_TYPE_ANNOTATION)
 G_DEFINE_TYPE_WITH_CODE (EvAnnotationText, ev_annotation_text, EV_TYPE_ANNOTATION,
 	 {
 		 G_IMPLEMENT_INTERFACE (EV_TYPE_ANNOTATION_MARKUP,
 					ev_annotation_text_markup_iface_init);
 	 });
+G_DEFINE_TYPE_WITH_CODE (EvAnnotationAttachment, ev_annotation_attachment, EV_TYPE_ANNOTATION,
+	 {
+		 G_IMPLEMENT_INTERFACE (EV_TYPE_ANNOTATION_MARKUP,
+					ev_annotation_attachment_markup_iface_init);
+	 });
 
 /* EvAnnotation */
 static void
@@ -109,17 +92,6 @@ ev_annotation_class_init (EvAnnotationClass *klass)
 	g_object_class->finalize = ev_annotation_finalize;
 }
 
-EvAnnotation *
-ev_annotation_text_new (EvPage *page)
-{
-	EvAnnotation *annot;
-
-	annot = EV_ANNOTATION (g_object_new (EV_TYPE_ANNOTATION_TEXT, NULL));
-	annot->page = g_object_ref (page);
-
-	return annot;
-}
-
 /* EvAnnotationMarkup */
 typedef struct {
 	gchar   *label;
@@ -130,7 +102,7 @@ typedef struct {
 } EvAnnotationMarkupProps;
 
 static void
-ev_annotation_markup_iface_base_init (EvAnnotationMarkupIface *iface)
+ev_annotation_markup_default_init (EvAnnotationMarkupInterface *iface)
 {
 	static gboolean initialized = FALSE;
 
@@ -383,7 +355,66 @@ ev_annotation_text_class_init (EvAnnotationTextClass *klass)
 }
 
 static void
-ev_annotation_text_markup_iface_init (EvAnnotationMarkupIface *iface)
+ev_annotation_text_markup_iface_init (EvAnnotationMarkupInterface *iface)
 {
 }
 
+EvAnnotation *
+ev_annotation_text_new (EvPage *page)
+{
+	EvAnnotation *annot;
+
+	annot = EV_ANNOTATION (g_object_new (EV_TYPE_ANNOTATION_TEXT, NULL));
+	annot->page = g_object_ref (page);
+
+	return annot;
+}
+
+/* EvAnnotationAttachment */
+static void
+ev_annotation_attachment_finalize (GObject *object)
+{
+	EvAnnotationAttachment *annot = EV_ANNOTATION_ATTACHMENT (object);
+
+	if (annot->attachment) {
+		g_object_unref (annot->attachment);
+		annot->attachment = NULL;
+	}
+
+	G_OBJECT_CLASS (ev_annotation_attachment_parent_class)->finalize (object);
+}
+
+static void
+ev_annotation_attachment_init (EvAnnotationAttachment *annot)
+{
+}
+
+static void
+ev_annotation_attachment_class_init (EvAnnotationAttachmentClass *klass)
+{
+	GObjectClass *g_object_class = G_OBJECT_CLASS (klass);
+
+	ev_annotation_markup_class_install_properties (g_object_class);
+
+	g_object_class->finalize = ev_annotation_attachment_finalize;
+}
+
+static void
+ev_annotation_attachment_markup_iface_init (EvAnnotationMarkupInterface *iface)
+{
+}
+
+EvAnnotation *
+ev_annotation_attachment_new (EvPage       *page,
+			      EvAttachment *attachment)
+{
+	EvAnnotation *annot;
+
+	g_return_val_if_fail (EV_IS_ATTACHMENT (attachment), NULL);
+
+	annot = EV_ANNOTATION (g_object_new (EV_TYPE_ANNOTATION_ATTACHMENT, NULL));
+	annot->page = g_object_ref (page);
+	EV_ANNOTATION_ATTACHMENT (annot)->attachment = g_object_ref (attachment);
+
+	return annot;
+}
