@@ -22,14 +22,12 @@
 #include <config.h>
 #include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include "gailmisc.h"
-#include "gailtextutil.h"
+#include <libgail-util/gail-util.h>
 
 #include "ev-selection.h"
 #include "ev-page-cache.h"
 #include "ev-view-accessible.h"
 #include "ev-view-private.h"
-#include "ev-mapping.h"
 
 #define EV_TYPE_VIEW_ACCESSIBLE      (ev_view_accessible_get_type ())
 #define EV_VIEW_ACCESSIBLE(obj)      (G_TYPE_CHECK_INSTANCE_CAST ((obj), EV_TYPE_VIEW_ACCESSIBLE, EvViewAccessible))
@@ -435,7 +433,7 @@ ev_view_accessible_get_character_extents (AtkText *text,
 
   if (coords == ATK_XY_SCREEN)
   {
-    gdk_window_get_origin (toplevel->window, &x_window, &y_window);
+    gdk_window_get_origin (gtk_widget_get_window (toplevel), &x_window, &y_window);
     *x += x_window;
     *y += y_window;
   }
@@ -498,7 +496,7 @@ ev_view_accessible_get_offset_at_point (AtkText *text,
 
   if (coords == ATK_XY_SCREEN)
   {
-    gdk_window_get_origin (toplevel->window, &x_window, &y_window);
+    gdk_window_get_origin (gtk_widget_get_window (toplevel), &x_window, &y_window);
     rx -= x_window;
     ry -= y_window;
   }
